@@ -36,20 +36,20 @@
 
 
 static void thunar_column_editor_finalize           (GObject                  *object);
-static void thunar_column_editor_help_clicked       (GtkWidget                *button,
-                                                     ThunarColumnEditor       *column_editor);
-static void thunar_column_editor_move_down          (GtkWidget                *button,
-                                                     ThunarColumnEditor       *column_editor);
-static void thunar_column_editor_move_up            (GtkWidget                *button,
-                                                     ThunarColumnEditor       *column_editor);
-static void thunar_column_editor_toggled            (GtkCellRendererToggle    *cell_renderer,
+static void thunar_column_editor_help_clicked       (ThunarColumnEditor       *column_editor,
+                                                     GtkWidget                *button);
+static void thunar_column_editor_move_down          (ThunarColumnEditor       *column_editor,
+                                                     GtkWidget                *button);
+static void thunar_column_editor_move_up            (ThunarColumnEditor       *column_editor,
+                                                     GtkWidget                *button);
+static void thunar_column_editor_toggled            (ThunarColumnEditor       *column_editor,
                                                      const gchar              *path_string,
-                                                     ThunarColumnEditor       *column_editor);
-static void thunar_column_editor_toggle_visibility  (GtkWidget                *button,
-                                                     ThunarColumnEditor       *column_editor);
+                                                     GtkCellRendererToggle    *cell_renderer);
+static void thunar_column_editor_toggle_visibility  (ThunarColumnEditor       *column_editor,
+                                                     GtkWidget                *button);
 static void thunar_column_editor_update_buttons     (ThunarColumnEditor       *column_editor);
-static void thunar_column_editor_use_defaults       (GtkWidget                *button,
-                                                     ThunarColumnEditor       *column_editor);
+static void thunar_column_editor_use_defaults       (ThunarColumnEditor       *column_editor,
+                                                     GtkWidget                *button);
 
 
 
@@ -65,6 +65,7 @@ struct _ThunarColumnEditor
   ThunarPreferences *preferences;
 
   ThunarColumnModel *column_model;
+  GtkTreeModelFilter *filter;
 
   GtkWidget         *tree_view;
   GtkWidget         *up_button;
@@ -90,12 +91,40 @@ thunar_column_editor_class_init (ThunarColumnEditorClass *klass)
 
 
 
+static gboolean
+thunar_column_visible_func (GtkTreeModel *model,
+                            GtkTreeIter  *iter,
+                            gpointer     data)
+{
+  gchar       *col_name;
+  const gchar *del_date_col_name;
+  gboolean     visible = TRUE;
+
+  /* fetching Date Deleted column name */
+  del_date_col_name = thunar_column_model_get_column_name (THUNAR_COLUMN_MODEL (model),
+                                                           THUNAR_COLUMN_DATE_DELETED);
+
+  /* fetching iter column name */
+  gtk_tree_model_get (model, iter, THUNAR_COLUMN_MODEL_COLUMN_NAME, &col_name, -1);
+
+  /* matching iter column name & Date Deleted column name */
+  if (g_strcmp0 (col_name, del_date_col_name) == 0)
+    visible = FALSE;
+
+  g_free (col_name);
+
+  return visible;
+}
+
+
+
 static void
 thunar_column_editor_init (ThunarColumnEditor *column_editor)
 {
   GtkTreeViewColumn *column;
   GtkTreeSelection  *selection;
   GtkCellRenderer   *renderer;
+  GtkTreeIter        childIter;
   GtkTreeIter        iter;
   GtkWidget         *separator;
   GtkWidget         *button;
@@ -117,6 +146,12 @@ thunar_column_editor_init (ThunarColumnEditor *column_editor)
   g_signal_connect_data (G_OBJECT (column_editor->column_model), "rows-reordered", G_CALLBACK (thunar_column_editor_update_buttons),
                          column_editor, NULL, G_CONNECT_AFTER | G_CONNECT_SWAPPED);
 
+  /* create a filter from the shared column model */
+  column_editor->filter = GTK_TREE_MODEL_FILTER (gtk_tree_model_filter_new (GTK_TREE_MODEL (column_editor->column_model), NULL));
+  gtk_tree_model_filter_set_visible_func (column_editor->filter,
+                                          (GtkTreeModelFilterVisibleFunc) thunar_column_visible_func,
+                                          NULL, NULL);
+
   /* setup the dialog */
   gtk_dialog_add_button (GTK_DIALOG (column_editor), _("_Close"), GTK_RESPONSE_CLOSE);
   gtk_dialog_set_default_response (GTK_DIALOG (column_editor), GTK_RESPONSE_CLOSE);
@@ -125,7 +160,7 @@ thunar_column_editor_init (ThunarColumnEditor *column_editor)
 
   /* add the "Help" button */
   button = gtk_button_new_with_mnemonic (_("_Help"));
-  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (thunar_column_editor_help_clicked), column_editor);
+  g_signal_connect_swapped (G_OBJECT (button), "clicked", G_CALLBACK (thunar_column_editor_help_clicked), column_editor);
   gtk_box_pack_start (GTK_BOX (exo_gtk_dialog_get_action_area (GTK_DIALOG (column_editor))), button, FALSE, FALSE, 0);
   gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (exo_gtk_dialog_get_action_area (GTK_DIALOG (column_editor))), button, TRUE);
   gtk_widget_show (button);
@@ -171,16 +206,15 @@ thunar_column_editor_init (ThunarColumnEditor *column_editor)
   gtk_widget_show (swin);
 
   /* create the tree view */
-  column_editor->tree_view = gtk_tree_view_new ();
+  column_editor->tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (column_editor->filter));
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (column_editor->tree_view), FALSE);
-  gtk_tree_view_set_model (GTK_TREE_VIEW (column_editor->tree_view), GTK_TREE_MODEL (column_editor->column_model));
   gtk_container_add (GTK_CONTAINER (swin), column_editor->tree_view);
   gtk_widget_show (column_editor->tree_view);
 
   /* append the toggle column */
   column = gtk_tree_view_column_new ();
   renderer = gtk_cell_renderer_toggle_new ();
-  g_signal_connect (G_OBJECT (renderer), "toggled", G_CALLBACK (thunar_column_editor_toggled), column_editor);
+  g_signal_connect_swapped (G_OBJECT (renderer), "toggled", G_CALLBACK (thunar_column_editor_toggled), column_editor);
   gtk_tree_view_column_pack_start (column, renderer, FALSE);
   gtk_tree_view_column_set_attributes (column, renderer,
                                        "active", THUNAR_COLUMN_MODEL_COLUMN_VISIBLE,
@@ -200,7 +234,7 @@ thunar_column_editor_init (ThunarColumnEditor *column_editor)
 
   /* create the "Move Up" button */
   column_editor->up_button = gtk_button_new_with_mnemonic (_("Move _Up"));
-  g_signal_connect (G_OBJECT (column_editor->up_button), "clicked", G_CALLBACK (thunar_column_editor_move_up), column_editor);
+  g_signal_connect_swapped (G_OBJECT (column_editor->up_button), "clicked", G_CALLBACK (thunar_column_editor_move_up), column_editor);
   gtk_grid_attach (GTK_GRID (grid), column_editor->up_button, 1, row, 1, 1);
   gtk_widget_show (column_editor->up_button);
 
@@ -214,7 +248,7 @@ thunar_column_editor_init (ThunarColumnEditor *column_editor)
 
   /* create the "Move Down" button */
   column_editor->down_button = gtk_button_new_with_mnemonic (_("Move Dow_n"));
-  g_signal_connect (G_OBJECT (column_editor->down_button), "clicked", G_CALLBACK (thunar_column_editor_move_down), column_editor);
+  g_signal_connect_swapped (G_OBJECT (column_editor->down_button), "clicked", G_CALLBACK (thunar_column_editor_move_down), column_editor);
   gtk_grid_attach (GTK_GRID (grid), column_editor->down_button, 1, row, 1, 1);
   gtk_widget_show (column_editor->down_button);
 
@@ -228,7 +262,7 @@ thunar_column_editor_init (ThunarColumnEditor *column_editor)
 
   /* create the "Show" button */
   column_editor->show_button = gtk_button_new_with_mnemonic (_("_Show"));
-  g_signal_connect (G_OBJECT (column_editor->show_button), "clicked", G_CALLBACK (thunar_column_editor_toggle_visibility), column_editor);
+  g_signal_connect_swapped (G_OBJECT (column_editor->show_button), "clicked", G_CALLBACK (thunar_column_editor_toggle_visibility), column_editor);
   gtk_grid_attach (GTK_GRID (grid), column_editor->show_button, 1, row, 1, 1);
   gtk_widget_show (column_editor->show_button);
 
@@ -237,7 +271,7 @@ thunar_column_editor_init (ThunarColumnEditor *column_editor)
 
   /* create the "Hide" button */
   column_editor->hide_button = gtk_button_new_with_mnemonic (_("Hi_de"));
-  g_signal_connect (G_OBJECT (column_editor->hide_button), "clicked", G_CALLBACK (thunar_column_editor_toggle_visibility), column_editor);
+  g_signal_connect_swapped (G_OBJECT (column_editor->hide_button), "clicked", G_CALLBACK (thunar_column_editor_toggle_visibility), column_editor);
   gtk_grid_attach (GTK_GRID (grid), column_editor->hide_button, 1, row, 1, 1);
   gtk_widget_show (column_editor->hide_button);
 
@@ -254,7 +288,7 @@ thunar_column_editor_init (ThunarColumnEditor *column_editor)
 
   /* create the "Use Default" button */
   button = gtk_button_new_with_mnemonic (_("Use De_fault"));
-  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (thunar_column_editor_use_defaults), column_editor);
+  g_signal_connect_swapped (G_OBJECT (button), "clicked", G_CALLBACK (thunar_column_editor_use_defaults), column_editor);
   gtk_grid_attach (GTK_GRID (grid), button, 1, row, 1, 1);
   gtk_widget_show (button);
 
@@ -305,8 +339,13 @@ thunar_column_editor_init (ThunarColumnEditor *column_editor)
   g_signal_connect_swapped (G_OBJECT (selection), "changed", G_CALLBACK (thunar_column_editor_update_buttons), column_editor);
 
   /* select the first item */
-  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (column_editor->column_model), &iter))
-    gtk_tree_selection_select_iter (selection, &iter);
+  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (column_editor->column_model), &childIter))
+    {
+      /* tree_view is created from filter & column_model is the child of filter
+       * hence, child Iter needs to be converted to parent Iter */
+      gtk_tree_model_filter_convert_child_iter_to_iter (column_editor->filter, &iter, &childIter);
+      gtk_tree_selection_select_iter (selection, &iter);
+    }
 
   /* grab focus to the tree view */
   gtk_widget_grab_focus (column_editor->tree_view);
@@ -332,8 +371,8 @@ thunar_column_editor_finalize (GObject *object)
 
 
 static void
-thunar_column_editor_help_clicked (GtkWidget          *button,
-                                   ThunarColumnEditor *column_editor)
+thunar_column_editor_help_clicked (ThunarColumnEditor *column_editor,
+                                   GtkWidget          *button)
 {
   _thunar_return_if_fail (THUNAR_IS_COLUMN_EDITOR (column_editor));
   _thunar_return_if_fail (GTK_IS_BUTTON (button));
@@ -348,13 +387,16 @@ thunar_column_editor_help_clicked (GtkWidget          *button,
 
 
 static void
-thunar_column_editor_move_down (GtkWidget          *button,
-                                ThunarColumnEditor *column_editor)
+thunar_column_editor_move_down (ThunarColumnEditor *column_editor,
+                                GtkWidget          *button)
 {
   GtkTreeSelection *selection;
   GtkTreeModel     *model;
+  GtkTreeModel     *childModel;
   GtkTreeIter       iter1;
   GtkTreeIter       iter2;
+  GtkTreeIter       childIter1;
+  GtkTreeIter       childIter2;
 
   _thunar_return_if_fail (THUNAR_IS_COLUMN_EDITOR (column_editor));
   _thunar_return_if_fail (GTK_IS_BUTTON (button));
@@ -368,21 +410,31 @@ thunar_column_editor_move_down (GtkWidget          *button,
 
       /* determine the next iterator and exchange the rows */
       if (gtk_tree_model_iter_next (model, &iter2))
-        thunar_column_model_exchange (THUNAR_COLUMN_MODEL (model), &iter1, &iter2);
+        {
+          /* tree view's model is made from GTK_TREE_MODEL_FILTER, hence fetching child model and child iter's */
+          gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (model), &childIter1, &iter1);
+          gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (model), &childIter2, &iter2);
+          childModel = gtk_tree_model_filter_get_model(GTK_TREE_MODEL_FILTER (model));
+
+          thunar_column_model_exchange (THUNAR_COLUMN_MODEL (childModel), &childIter1, &childIter2);
+        }
     }
 }
 
 
 
 static void
-thunar_column_editor_move_up (GtkWidget          *button,
-                              ThunarColumnEditor *column_editor)
+thunar_column_editor_move_up (ThunarColumnEditor *column_editor,
+                              GtkWidget          *button)
 {
   GtkTreeSelection *selection;
   GtkTreeModel     *model;
+  GtkTreeModel     *childModel;
   GtkTreePath      *path;
   GtkTreeIter       iter1;
   GtkTreeIter       iter2;
+  GtkTreeIter       childIter1;
+  GtkTreeIter       childIter2;
 
   _thunar_return_if_fail (THUNAR_IS_COLUMN_EDITOR (column_editor));
   _thunar_return_if_fail (GTK_IS_BUTTON (button));
@@ -398,8 +450,13 @@ thunar_column_editor_move_up (GtkWidget          *button,
           /* advance to the prev path */
           if (gtk_tree_path_prev (path) && gtk_tree_model_get_iter (model, &iter2, path))
             {
+              /* tree view's model is made from GTK_TREE_MODEL_FILTER, hence fetching child model and child iter's */
+              gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (model), &childIter1, &iter1);
+              gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (model), &childIter2, &iter2);
+              childModel = gtk_tree_model_filter_get_model(GTK_TREE_MODEL_FILTER (model));
+
               /* exchange the rows */
-              thunar_column_model_exchange (THUNAR_COLUMN_MODEL (model), &iter1, &iter2);
+              thunar_column_model_exchange (THUNAR_COLUMN_MODEL (childModel), &childIter1, &childIter2);
             }
 
           /* release the path */
@@ -411,13 +468,14 @@ thunar_column_editor_move_up (GtkWidget          *button,
 
 
 static void
-thunar_column_editor_toggled (GtkCellRendererToggle *cell_renderer,
+thunar_column_editor_toggled (ThunarColumnEditor    *column_editor,
                               const gchar           *path_string,
-                              ThunarColumnEditor    *column_editor)
+                              GtkCellRendererToggle *cell_renderer)
 {
   ThunarColumn column;
   GtkTreePath *path;
   GtkTreeIter  iter;
+  GtkTreeIter  child_iter;
   gboolean     visible;
 
   _thunar_return_if_fail (GTK_IS_CELL_RENDERER_TOGGLE (cell_renderer));
@@ -426,10 +484,12 @@ thunar_column_editor_toggled (GtkCellRendererToggle *cell_renderer,
 
   /* determine the tree path for the string */
   path = gtk_tree_path_new_from_string (path_string);
-  if (gtk_tree_model_get_iter (GTK_TREE_MODEL (column_editor->column_model), &iter, path))
+  if (gtk_tree_model_get_iter (GTK_TREE_MODEL (column_editor->filter), &iter, path))
     {
+      gtk_tree_model_filter_convert_iter_to_child_iter (column_editor->filter, &child_iter, &iter);
+
       /* determine the column for the iterator... */
-      column = thunar_column_model_get_column_for_iter (column_editor->column_model, &iter);
+      column = thunar_column_model_get_column_for_iter (column_editor->column_model, &child_iter);
 
       /* ...determine the existing visbility setting... */
       visible = thunar_column_model_get_column_visible (column_editor->column_model, column);
@@ -443,12 +503,14 @@ thunar_column_editor_toggled (GtkCellRendererToggle *cell_renderer,
 
 
 static void
-thunar_column_editor_toggle_visibility (GtkWidget          *button,
-                                        ThunarColumnEditor *column_editor)
+thunar_column_editor_toggle_visibility (ThunarColumnEditor *column_editor,
+                                        GtkWidget          *button)
 {
   GtkTreeSelection *selection;
   ThunarColumn      column;
+  GtkTreeModel     *model;
   GtkTreeIter       iter;
+  GtkTreeIter       childIter;
   gboolean          visible;
 
   _thunar_return_if_fail (THUNAR_IS_COLUMN_EDITOR (column_editor));
@@ -456,10 +518,13 @@ thunar_column_editor_toggle_visibility (GtkWidget          *button,
 
   /* determine the selected tree iterator */
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (column_editor->tree_view));
-  if (gtk_tree_selection_get_selected (selection, NULL, &iter))
+  if (gtk_tree_selection_get_selected (selection, &model, &iter))
     {
+      /* tree view's model is made from GTK_TREE_MODEL_FILTER, hence fetching child iter */
+      gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (model), &childIter, &iter);
+
       /* determine the column for the iterator... */
-      column = thunar_column_model_get_column_for_iter (column_editor->column_model, &iter);
+      column = thunar_column_model_get_column_for_iter (column_editor->column_model, &childIter);
 
       /* ...determine the existing visbility setting... */
       visible = thunar_column_model_get_column_visible (column_editor->column_model, column);
@@ -488,6 +553,7 @@ thunar_column_editor_update_buttons (ThunarColumnEditor *column_editor)
     {
       /* determine the tree path for the iter */
       path = gtk_tree_model_get_path (model, &iter);
+
       if (G_UNLIKELY (path == NULL))
         return;
 
@@ -517,8 +583,8 @@ thunar_column_editor_update_buttons (ThunarColumnEditor *column_editor)
 
 
 static void
-thunar_column_editor_use_defaults (GtkWidget          *button,
-                                   ThunarColumnEditor *column_editor)
+thunar_column_editor_use_defaults (ThunarColumnEditor *column_editor,
+                                   GtkWidget          *button)
 {
   static const gchar *PROPERTY_NAMES[] =
   {
