@@ -86,6 +86,14 @@ static gboolean thunar_dbus_service_display_chooser_dialog      (ThunarDBusFileM
                                                                  const gchar            *display,
                                                                  const gchar            *startup_id,
                                                                  ThunarDBusService      *dbus_service);
+static gboolean thunar_dbus_service_display_app_chooser_dialog  (ThunarDBusFileManager  *object,
+                                                                 GDBusMethodInvocation  *invocation,
+                                                                 const gchar            *uri,
+                                                                 gboolean                open,
+                                                                 gboolean                preselect_default_checkbox,
+                                                                 const gchar            *display,
+                                                                 const gchar            *startup_id,
+                                                                 ThunarDBusService      *dbus_service);
 static gboolean thunar_dbus_service_display_folder              (ThunarDBusFileManager  *object,
                                                                  GDBusMethodInvocation  *invocation,
                                                                  const gchar            *uri,
@@ -305,6 +313,7 @@ thunar_dbus_service_init (ThunarDBusService *dbus_service)
 
   connect_signals_multiple (dbus_service->file_manager, dbus_service,
                             "handle-display-chooser-dialog", thunar_dbus_service_display_chooser_dialog,
+                            "handle-display-application-chooser-dialog", thunar_dbus_service_display_app_chooser_dialog,
                             "handle-display-folder", thunar_dbus_service_display_folder,
                             "handle-display-folder-and-select", thunar_dbus_service_display_folder_and_select,
                             "handle-display-file-properties", thunar_dbus_service_display_file_properties,
@@ -432,6 +441,9 @@ thunar_dbus_service_trash_bin_changed (ThunarDBusService *dbus_service,
 
 
 
+/**
+ * TODO: Method to be removed after thunar 4.18 got released (dbus call is marked as deprecated). Use thunar_dbus_service_display_app_chooser_dialog instead.
+ */
 static gboolean
 thunar_dbus_service_display_chooser_dialog (ThunarDBusFileManager  *object,
                                             GDBusMethodInvocation  *invocation,
@@ -468,6 +480,43 @@ out:
 
 
 
+static gboolean
+thunar_dbus_service_display_app_chooser_dialog (ThunarDBusFileManager  *object,
+                                                GDBusMethodInvocation  *invocation,
+                                                const gchar            *uri,
+                                                gboolean                open,
+                                                gboolean                preselect_default_checkbox,
+                                                const gchar            *display,
+                                                const gchar            *startup_id,
+                                                ThunarDBusService      *dbus_service)
+{
+  ThunarFile *file;
+  GdkScreen  *screen;
+  GError     *error = NULL;
+
+  /* parse uri and display parameters */
+  if (!thunar_dbus_service_parse_uri_and_display (dbus_service, uri, display, &file, &screen, &error))
+    goto out;
+
+  /* popup the chooser dialog */
+  /* TODO use the startup id! */
+  thunar_show_chooser_dialog (screen, file, open, preselect_default_checkbox);
+
+  /* cleanup */
+  g_object_unref (G_OBJECT (screen));
+  g_object_unref (G_OBJECT (file));
+
+out:
+  if (error)
+    g_dbus_method_invocation_take_error (invocation, error);
+  else
+    thunar_dbus_file_manager_complete_display_chooser_dialog (object, invocation);
+
+  return TRUE;
+}
+
+
+
 static gboolean
 thunar_dbus_service_display_folder (ThunarDBusFileManager  *object,
                                     GDBusMethodInvocation  *invocation,
@@ -865,7 +914,7 @@ thunar_dbus_service_move_to_trash (ThunarDBusTrash        *object,
         {
           /* tell the application to move the specified files to the trash */
           application = thunar_application_get ();
-          thunar_application_trash (application, screen, file_list);
+          thunar_application_trash (application, screen, file_list, THUNAR_OPERATION_LOG_NO_OPERATIONS);
           g_object_unref (application);
         }
 
@@ -1028,7 +1077,7 @@ thunar_dbus_service_rename_file (ThunarDBusFileManager  *object,
 
   /* popup a new window for the folder */
   application = thunar_application_get ();
-  thunar_application_rename_file (application, file, screen, startup_id);
+  thunar_application_rename_file (application, file, screen, startup_id, THUNAR_OPERATION_LOG_NO_OPERATIONS);
   g_object_unref (G_OBJECT (application));
 
   /* cleanup */
@@ -1070,7 +1119,7 @@ thunar_dbus_service_create_file (ThunarDBusFileManager  *object,
 
   /* popup a new window for the folder */
   application = thunar_application_get ();
-  thunar_application_create_file (application, file, content_type, screen, startup_id);
+  thunar_application_create_file (application, file, content_type, screen, startup_id, THUNAR_OPERATION_LOG_NO_OPERATIONS);
   g_object_unref (G_OBJECT (application));
 
   /* cleanup */
@@ -1114,7 +1163,7 @@ thunar_dbus_service_create_file_from_template (ThunarDBusFileManager  *object,
 
   /* popup a new window for the folder */
   application = thunar_application_get ();
-  thunar_application_create_file_from_template (application, file, template_file, screen, startup_id);
+  thunar_application_create_file_from_template (application, file, template_file, screen, startup_id, THUNAR_OPERATION_LOG_NO_OPERATIONS);
   g_object_unref (G_OBJECT (application));
 
   /* cleanup */
@@ -1231,21 +1280,25 @@ thunar_dbus_service_transfer_files (ThunarDBusTransferMode transfer_mode,
             case THUNAR_DBUS_TRANSFER_MODE_COPY_TO:
               thunar_application_copy_to (application, screen,
                                           source_file_list, target_file_list,
+                                          THUNAR_OPERATION_LOG_NO_OPERATIONS,
                                           NULL);
               break;
             case THUNAR_DBUS_TRANSFER_MODE_COPY_INTO:
               thunar_application_copy_into (application, screen,
                                             source_file_list, target_file_list->data,
+                                            THUNAR_OPERATION_LOG_NO_OPERATIONS,
                                             NULL);
               break;
             case THUNAR_DBUS_TRANSFER_MODE_MOVE_INTO:
               thunar_application_move_into (application, screen,
                                             source_file_list, target_file_list->data,
+                                            THUNAR_OPERATION_LOG_NO_OPERATIONS,
                                             NULL);
               break;
             case THUNAR_DBUS_TRANSFER_MODE_LINK_INTO:
               thunar_application_link_into (application, screen,
                                             source_file_list, target_file_list->data,
+                                            THUNAR_OPERATION_LOG_NO_OPERATIONS,
                                             NULL);
               break;
             }
