@@ -640,3 +640,75 @@ thunar_status_bar_info_check_active (guint               info,
 {
   return (info & mask) > 0 ? TRUE : FALSE;
 }
+
+
+
+GType
+thunar_job_operation_kind_get_type (void)
+{
+  static GType type = G_TYPE_INVALID;
+
+  if (G_UNLIKELY (type == G_TYPE_INVALID))
+    {
+      static const GEnumValue values[] =
+      {
+        { THUNAR_JOB_OPERATION_KIND_COPY,         "THUNAR_JOB_OPERATION_KIND_COPY",          N_("Copy"),  },
+        { THUNAR_JOB_OPERATION_KIND_MOVE,         "THUNAR_JOB_OPERATION_KIND_MOVE",          N_("Move") },
+        { THUNAR_JOB_OPERATION_KIND_RENAME,       "THUNAR_JOB_OPERATION_KIND_RENAME",        N_("Rename") },
+        { THUNAR_JOB_OPERATION_KIND_CREATE_FILE,  "THUNAR_JOB_OPERATION_KIND_CREATE_FILE",   N_("Create File") },
+        { THUNAR_JOB_OPERATION_KIND_CREATE_FOLDER,"THUNAR_JOB_OPERATION_KIND_CREATE_FOLDER", N_("Create Folder") },
+        { THUNAR_JOB_OPERATION_KIND_DELETE,       "THUNAR_JOB_OPERATION_KIND_DELETE",        N_("Delete (opposite of create)") },
+        { THUNAR_JOB_OPERATION_KIND_TRASH,        "THUNAR_JOB_OPERATION_KIND_TRASH",         N_("Trash") },
+        { THUNAR_JOB_OPERATION_KIND_RESTORE,      "THUNAR_JOB_OPERATION_KIND_RESTORE",       N_("Restore (opposite of trash)") },
+        { THUNAR_JOB_OPERATION_KIND_LINK,         "THUNAR_JOB_OPERATION_KIND_LINK",          N_("Link") },
+        { THUNAR_JOB_OPERATION_KIND_UNLINK,       "THUNAR_JOB_OPERATION_KIND_UNLINK",        N_("Unlink") },
+        { 0,                                      NULL,                                      NULL }
+      };
+
+      type = g_enum_register_static ("ThunarJobOperationKind", values);
+    }
+
+  return type;
+}
+
+GType
+thunar_operation_log_mode_get_type (void)
+{
+  static GType type = G_TYPE_INVALID;
+
+  if (G_UNLIKELY (type == G_TYPE_INVALID))
+    {
+      static const GEnumValue values[] =
+      {
+        { THUNAR_OPERATION_LOG_OPERATIONS,      "THUNAR_OPERATION_LOG_OPERATIONS",      N_("Log operations") },
+        { THUNAR_OPERATION_LOG_NO_OPERATIONS,   "THUNAR_OPERATION_LOG_NO_OPERATIONS",   N_("Log no operations") },
+        { THUNAR_OPERATION_LOG_ONLY_TIMESTAMPS, "THUNAR_OPERATION_LOG_ONLY_TIMESTAMPS", N_("Log only timestamps") },
+        { 0,                                    NULL,                                   NULL }
+      };
+
+      type = g_enum_register_static ("ThunarOperationLogMode", values);
+    }
+
+  return type;
+}
+
+GType
+thunar_image_preview_mode_get_type (void)
+{
+  static GType type = G_TYPE_INVALID;
+
+  if (G_UNLIKELY (type == G_TYPE_INVALID))
+    {
+      static const GEnumValue values[] =
+      {
+        { THUNAR_IMAGE_PREVIEW_MODE_STANDALONE, "THUNAR_IMAGE_PREVIEW_MODE_STANDALONE", N_("Standalone Image Preview") },
+        { THUNAR_IMAGE_PREVIEW_MODE_EMBEDDED,   "THUNAR_IMAGE_PREVIEW_MODE_EMBEDDED",   N_("Embedded Image Preview") },
+        { 0,                                    NULL,                                   NULL }
+      };
+
+      type = g_enum_register_static ("ThunarImagePreviewMode", values);
+    }
+
+  return type;
+}
+
