@@ -188,7 +188,9 @@ typedef enum
   THUNAR_THUMBNAIL_SIZE_NORMAL,
   THUNAR_THUMBNAIL_SIZE_LARGE,
   THUNAR_THUMBNAIL_SIZE_X_LARGE,
-  THUNAR_THUMBNAIL_SIZE_XX_LARGE
+  THUNAR_THUMBNAIL_SIZE_XX_LARGE,
+
+  THUNAR_THUMBNAIL_SIZE_DEFAULT = -1
 } ThunarThumbnailSize;
 
 GType       thunar_thumbnail_size_get_type (void)                               G_GNUC_CONST;
@@ -425,6 +427,59 @@ guint    thunar_status_bar_info_toggle_bit    (guint               info,
 gboolean thunar_status_bar_info_check_active  (guint               info,
                                                ThunarStatusBarInfo mask);
 
+
+
+#define THUNAR_TYPE_JOB_OPERATION_KIND (thunar_job_operation_kind_get_type ())
+
+typedef enum
+{
+  THUNAR_JOB_OPERATION_KIND_COPY,
+  THUNAR_JOB_OPERATION_KIND_MOVE,
+  THUNAR_JOB_OPERATION_KIND_RENAME,
+  THUNAR_JOB_OPERATION_KIND_CREATE_FILE,
+  THUNAR_JOB_OPERATION_KIND_CREATE_FOLDER,
+  THUNAR_JOB_OPERATION_KIND_DELETE,
+  THUNAR_JOB_OPERATION_KIND_TRASH,
+  THUNAR_JOB_OPERATION_KIND_RESTORE,
+  THUNAR_JOB_OPERATION_KIND_LINK,
+  THUNAR_JOB_OPERATION_KIND_UNLINK
+} ThunarJobOperationKind;
+
+GType thunar_job_operation_kind_get_type (void) G_GNUC_CONST;
+
+/**
+ * ThunarOperationLogMode:
+ *
+ * Specify control logging for operations.
+ **/
+
+#define THUNAR_TYPE_OPERATION_LOG_MODE (thunar_operation_log_mode_get_type ())
+
+typedef enum
+{
+  THUNAR_OPERATION_LOG_NO_OPERATIONS,
+  THUNAR_OPERATION_LOG_OPERATIONS,
+  THUNAR_OPERATION_LOG_ONLY_TIMESTAMPS,
+} ThunarOperationLogMode;
+
+GType thunar_operation_log_mode_get_type (void) G_GNUC_CONST;
+
+/**
+ * ThunarImagePreviewMode:
+ *
+ * Specify control logging for operations.
+ **/
+
+#define THUNAR_TYPE_IMAGE_PREVIEW_MODE (thunar_image_preview_mode_get_type ())
+
+typedef enum
+{
+  THUNAR_IMAGE_PREVIEW_MODE_STANDALONE,
+  THUNAR_IMAGE_PREVIEW_MODE_EMBEDDED,
+} ThunarImagePreviewMode;
+
+GType thunar_image_preview_mode_get_type (void) G_GNUC_CONST;
+
 G_END_DECLS;
 
 #endif /* !__THUNAR_ENUM_TYPES_H__ */
