@@ -246,6 +246,8 @@ gboolean          thunar_file_set_custom_icon            (ThunarFile
 
 const gchar     *thunar_file_get_thumbnail_path          (ThunarFile              *file,
                                                           ThunarThumbnailSize      thumbnail_size);
+gchar           *thunar_file_get_thumbnail_path_forced   (ThunarFile              *file,
+                                                          ThunarThumbnailSize      thumbnail_size);
 ThunarFileThumbState thunar_file_get_thumb_state         (const ThunarFile        *file);
 void             thunar_file_set_thumb_state             (ThunarFile              *file,
                                                           ThunarFileThumbState     state);
@@ -287,6 +289,8 @@ void              thunar_file_set_metadata_setting       (ThunarFile
                                                           const gchar            *setting_name,
                                                           const gchar            *setting_value,
                                                           gboolean                async);
+void              thunar_file_clear_metadata_setting     (ThunarFile             *file,
+                                                          const gchar            *setting_name);
 void              thunar_file_clear_directory_specific_settings (ThunarFile      *file);
 gboolean          thunar_file_has_directory_specific_settings   (ThunarFile      *file);
 
