@@ -657,28 +657,28 @@ thunar_g_file_get_free_space (GFile   *file,
 gchar *
 thunar_g_file_get_free_space_string (GFile *file, gboolean file_size_binary)
 {
-  gchar             *fs_free_str;
-  gchar             *fs_size_str;
-  guint64            fs_free;
-  guint64            fs_size;
-  gchar             *fs_string = NULL;
+  gchar             *fs_size_free_str;
+  gchar             *fs_size_used_str;
+  guint64            fs_size_free;
+  guint64            fs_size_total;
+  gchar             *free_space_string = NULL;
 
   _thunar_return_val_if_fail (G_IS_FILE (file), NULL);
 
-  if (thunar_g_file_get_free_space (file, &fs_free, &fs_size)
-      && fs_size > 0)
+  if (thunar_g_file_get_free_space (file, &fs_size_free, &fs_size_total) && fs_size_total > 0)
     {
-      fs_free_str = g_format_size_full (fs_free, file_size_binary ? G_FORMAT_SIZE_IEC_UNITS : G_FORMAT_SIZE_DEFAULT);
-      fs_size_str = g_format_size_full (fs_size, file_size_binary ? G_FORMAT_SIZE_IEC_UNITS : G_FORMAT_SIZE_DEFAULT);
-      /* free disk space string */
-      fs_string = g_strdup_printf (_("%s of %s free (%d%% used)"),
-                                   fs_free_str, fs_size_str,
-                                   (gint) ((fs_size - fs_free) * 100 / fs_size));
-      g_free (fs_free_str);
-      g_free (fs_size_str);
+      fs_size_free_str  = g_format_size_full (fs_size_free,  file_size_binary ? G_FORMAT_SIZE_IEC_UNITS : G_FORMAT_SIZE_DEFAULT);
+      fs_size_used_str =  g_format_size_full (fs_size_total - fs_size_free, file_size_binary ? G_FORMAT_SIZE_IEC_UNITS : G_FORMAT_SIZE_DEFAULT);
+
+      free_space_string = g_strdup_printf (_("%s used (%d%%)  |  %s free (%d%%)"),
+                                   fs_size_used_str, (gint) ((fs_size_total - fs_size_free) * 100 / fs_size_total),
+                                   fs_size_free_str, (gint) (fs_size_free * 100 / fs_size_total));
+
+      g_free (fs_size_free_str);
+      g_free (fs_size_used_str);
     }
 
-  return fs_string;
+  return free_space_string;
 }
 
 
