@@ -121,17 +121,19 @@ _thunar_io_jobs_create (ThunarJob  *job,
                         GArray     *param_values,
                         GError    **error)
 {
-  GFileOutputStream *stream;
-  ThunarJobResponse  response = THUNAR_JOB_RESPONSE_CANCEL;
-  GFileInfo         *info;
-  GError            *err = NULL;
-  GList             *file_list;
-  GList             *lp;
-  gchar             *base_name;
-  gchar             *display_name;
-  guint              n_processed = 0;
-  GFile             *template_file;
-  GFileInputStream  *template_stream = NULL;
+  GFileOutputStream      *stream;
+  ThunarJobResponse       response = THUNAR_JOB_RESPONSE_CANCEL;
+  GFileInfo              *info;
+  GError                 *err = NULL;
+  GList                  *file_list;
+  GList                  *lp;
+  gchar                  *base_name;
+  gchar                  *display_name;
+  guint                   n_processed = 0;
+  GFile                  *template_file;
+  GFileInputStream       *template_stream = NULL;
+  ThunarJobOperation     *operation = NULL;
+  ThunarOperationLogMode  log_mode;
 
   _thunar_return_val_if_fail (THUNAR_IS_JOB (job), FALSE);
   _thunar_return_val_if_fail (param_values != NULL, FALSE);
@@ -157,6 +159,11 @@ _thunar_io_jobs_create (ThunarJob  *job,
         }
     }
 
+  log_mode = thunar_job_get_log_mode (job);
+
+  if (log_mode == THUNAR_OPERATION_LOG_OPERATIONS)
+    operation = thunar_job_operation_new (THUNAR_JOB_OPERATION_KIND_CREATE_FILE);
+
   /* iterate over all files in the list */
   for (lp = file_list;
        err == NULL && lp != NULL && !exo_job_is_cancelled (EXO_JOB (job));
@@ -247,6 +254,11 @@ _thunar_io_jobs_create (ThunarJob  *job,
         }
       else
         {
+
+          /* remember the file for possible undo */
+          if (log_mode == THUNAR_OPERATION_LOG_OPERATIONS)
+               thunar_job_operation_add (operation, template_file, lp->data);
+
           if (template_stream != NULL)
             {
               /* write the template into the new file */
@@ -268,12 +280,24 @@ _thunar_io_jobs_create (ThunarJob  *job,
   if (err != NULL)
     {
       g_propagate_error (error, err);
+      if (operation != NULL)
+        g_object_unref (operation);
       return FALSE;
     }
 
   /* check if the job was cancelled */
   if (exo_job_is_cancelled (EXO_JOB (job)))
-    return FALSE;
+    {
+      if (operation != NULL)
+        g_object_unref (operation);
+      return FALSE;
+    }
+
+  if (log_mode == THUNAR_OPERATION_LOG_OPERATIONS)
+    {
+      thunar_job_operation_history_commit (operation);
+      g_object_unref (operation);
+    }
 
   /* emit the "new-files" signal with the given file list */
   thunar_job_new_files (THUNAR_JOB (job), file_list);
@@ -299,14 +323,16 @@ _thunar_io_jobs_mkdir (ThunarJob  *job,
                        GArray     *param_values,
                        GError    **error)
 {
-  ThunarJobResponse response;
-  GFileInfo        *info;
-  GError           *err = NULL;
-  GList            *file_list;
-  GList            *lp;
-  gchar            *base_name;
-  gchar            *display_name;
-  guint             n_processed = 0;
+  ThunarJobResponse       response;
+  GFileInfo              *info;
+  GError                 *err = NULL;
+  GList                  *file_list;
+  GList                  *lp;
+  gchar                  *base_name;
+  gchar                  *display_name;
+  guint                  n_processed = 0;
+  ThunarJobOperation     *operation = NULL;
+  ThunarOperationLogMode  log_mode;
 
   _thunar_return_val_if_fail (THUNAR_IS_JOB (job), FALSE);
   _thunar_return_val_if_fail (param_values != NULL, FALSE);
@@ -318,6 +344,11 @@ _thunar_io_jobs_mkdir (ThunarJob  *job,
   /* we know the total list of files to process */
   thunar_job_set_total_files (THUNAR_JOB (job), file_list);
 
+  log_mode = thunar_job_get_log_mode (job);
+
+  if (log_mode == THUNAR_OPERATION_LOG_OPERATIONS)
+    operation = thunar_job_operation_new (THUNAR_JOB_OPERATION_KIND_CREATE_FOLDER);
+
   for (lp = file_list;
        err == NULL && lp != NULL && !exo_job_is_cancelled (EXO_JOB (job));
        lp = lp->next,  n_processed++)
@@ -400,19 +431,36 @@ _thunar_io_jobs_mkdir (ThunarJob  *job,
               if (response == THUNAR_JOB_RESPONSE_RETRY)
                 goto again;
             }
-        }
-    }
+        } /* end try creation */
+
+      /* remember the file for possible undo */
+      if (log_mode == THUNAR_OPERATION_LOG_OPERATIONS)
+          thunar_job_operation_add (operation, NULL, lp->data);
+
+    } /* end for all files */
 
   /* check if we have failed */
   if (err != NULL)
     {
       g_propagate_error (error, err);
+      if (operation != NULL)
+        g_object_unref (operation);
       return FALSE;
     }
 
   /* check if the job was cancelled */
   if (exo_job_is_cancelled (EXO_JOB (job)))
-    return FALSE;
+    {
+      if (operation != NULL)
+        g_object_unref (operation);
+      return FALSE;
+    }
+
+  if (log_mode == THUNAR_OPERATION_LOG_OPERATIONS)
+    {
+      thunar_job_operation_history_commit (operation);
+      g_object_unref (operation);
+    }
 
   /* emit the "new-files" signal with the given file list */
   thunar_job_new_files (THUNAR_JOB (job), file_list);
@@ -713,16 +761,18 @@ _thunar_io_jobs_link (ThunarJob  *job,
                       GArray     *param_values,
                       GError    **error)
 {
-  ThunarThumbnailCache *thumbnail_cache;
-  ThunarApplication    *application;
-  GError               *err = NULL;
-  GFile                *real_target_file;
-  GList                *new_files_list = NULL;
-  GList                *source_file_list;
-  GList                *sp;
-  GList                *target_file_list;
-  GList                *tp;
-  guint                 n_processed = 0;
+  ThunarThumbnailCache   *thumbnail_cache;
+  ThunarApplication      *application;
+  GError                 *err = NULL;
+  GFile                  *real_target_file;
+  GList                  *new_files_list = NULL;
+  GList                  *source_file_list;
+  GList                  *sp;
+  GList                  *target_file_list;
+  GList                  *tp;
+  guint                   n_processed = 0;
+  ThunarJobOperation     *operation = NULL;
+  ThunarOperationLogMode  log_mode;
 
   _thunar_return_val_if_fail (THUNAR_IS_JOB (job), FALSE);
   _thunar_return_val_if_fail (param_values != NULL, FALSE);
@@ -740,6 +790,11 @@ _thunar_io_jobs_link (ThunarJob  *job,
   thumbnail_cache = thunar_application_get_thumbnail_cache (application);
   g_object_unref (application);
 
+  log_mode = thunar_job_get_log_mode (job);
+
+  if (log_mode == THUNAR_OPERATION_LOG_OPERATIONS)
+    operation = thunar_job_operation_new (THUNAR_JOB_OPERATION_KIND_LINK);
+
   /* process all files */
   for (sp = source_file_list, tp = target_file_list;
        err == NULL && sp != NULL && tp != NULL;
@@ -766,6 +821,9 @@ _thunar_io_jobs_link (ThunarJob  *job,
               thunar_thumbnail_cache_copy_file (thumbnail_cache, sp->data,
                                                 real_target_file);
 
+              /* remember the file for possible undo */
+              if (log_mode == THUNAR_OPERATION_LOG_OPERATIONS)
+                thunar_job_operation_add (operation, sp->data, real_target_file);
             }
 
           /* release the real target file */
@@ -776,6 +834,12 @@ _thunar_io_jobs_link (ThunarJob  *job,
   /* release the thumbnail cache */
   g_object_unref (thumbnail_cache);
 
+  if (log_mode == THUNAR_OPERATION_LOG_OPERATIONS)
+    {
+      thunar_job_operation_history_commit (operation);
+      g_object_unref (operation);
+    }
+
   if (err != NULL)
     {
       thunar_g_list_free_full (new_files_list);
@@ -812,12 +876,14 @@ _thunar_io_jobs_trash (ThunarJob  *job,
                        GArray     *param_values,
                        GError    **error)
 {
-  ThunarThumbnailCache *thumbnail_cache;
-  ThunarApplication    *application;
-  ThunarJobResponse     response;
-  GError               *err = NULL;
-  GList                *file_list;
-  GList                *lp;
+  ThunarThumbnailCache   *thumbnail_cache;
+  ThunarApplication      *application;
+  ThunarJobOperation     *operation = NULL;
+  ThunarJobResponse       response;
+  ThunarOperationLogMode  log_mode;
+  GError                 *err = NULL;
+  GList                  *file_list;
+  GList                  *lp;
 
   _thunar_return_val_if_fail (THUNAR_IS_JOB (job), FALSE);
   _thunar_return_val_if_fail (param_values != NULL, FALSE);
@@ -834,6 +900,11 @@ _thunar_io_jobs_trash (ThunarJob  *job,
   thumbnail_cache = thunar_application_get_thumbnail_cache (application);
   g_object_unref (application);
 
+  log_mode = thunar_job_get_log_mode (job);
+
+  if (log_mode != THUNAR_OPERATION_LOG_NO_OPERATIONS)
+    operation = thunar_job_operation_new (THUNAR_JOB_OPERATION_KIND_TRASH);
+
   for (lp = file_list; err == NULL && lp != NULL; lp = lp->next)
     {
       _thunar_assert (G_IS_FILE (lp->data));
@@ -854,6 +925,9 @@ _thunar_io_jobs_trash (ThunarJob  *job,
             _tij_delete_file (lp->data, exo_job_get_cancellable (EXO_JOB (job)), &err);
         }
 
+      if (err == NULL && log_mode != THUNAR_OPERATION_LOG_NO_OPERATIONS)
+          thunar_job_operation_add (operation, lp->data, NULL);
+
       /* update the thumbnail cache */
       thunar_thumbnail_cache_cleanup_file (thumbnail_cache, lp->data);
     }
@@ -861,6 +935,18 @@ _thunar_io_jobs_trash (ThunarJob  *job,
   /* release the thumbnail cache */
   g_object_unref (thumbnail_cache);
 
+  if (log_mode == THUNAR_OPERATION_LOG_OPERATIONS)
+    {
+      thunar_job_operation_history_commit (operation);
+      g_object_unref (operation);
+    }
+  else if (log_mode == THUNAR_OPERATION_LOG_ONLY_TIMESTAMPS)
+    {
+      /* only required for 'redo' operation, in order to update the timestamps of the original trash operation */
+      thunar_job_operation_history_update_trash_timestamps (operation);
+      g_object_unref (operation);
+    }
+
   if (err != NULL)
     {
       g_propagate_error (error, err);
@@ -1297,23 +1383,30 @@ _thunar_io_jobs_rename (ThunarJob  *job,
                         GArray     *param_values,
                         GError    **error)
 {
-  const gchar *display_name;
-  ThunarFile  *file;
-  GError      *err = NULL;
+  const gchar           *display_name;
+  ThunarFile            *file;
+  GError                *err = NULL;
+  gchar                 *old_file_uri;
+  ThunarOperationLogMode log_mode;
+  ThunarJobOperation     *operation;
 
   _thunar_return_val_if_fail (THUNAR_IS_JOB (job), FALSE);
   _thunar_return_val_if_fail (param_values != NULL, FALSE);
-  _thunar_return_val_if_fail (param_values->len == 2, FALSE);
+  _thunar_return_val_if_fail (param_values->len == 3, FALSE);
   _thunar_return_val_if_fail (G_VALUE_HOLDS (&g_array_index (param_values, GValue, 0), THUNAR_TYPE_FILE), FALSE);
   _thunar_return_val_if_fail (G_VALUE_HOLDS_STRING (&g_array_index (param_values, GValue, 1)), FALSE);
+  _thunar_return_val_if_fail (G_VALUE_HOLDS_ENUM (&g_array_index (param_values, GValue, 2)), FALSE);
   _thunar_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   if (exo_job_set_error_if_cancelled (EXO_JOB (job), error))
     return FALSE;
 
-  /* determine the file and display name */
+  /* determine the file, display name and log mode */
   file = g_value_get_object (&g_array_index (param_values, GValue, 0));
   display_name = g_value_get_string (&g_array_index (param_values, GValue, 1));
+  log_mode = g_value_get_enum (&g_array_index (param_values, GValue, 2));
+
+  old_file_uri = g_file_get_uri (thunar_file_get_file (file));
 
   /* try to rename the file */
   if (thunar_file_rename (file, display_name, exo_job_get_cancellable (EXO_JOB (job)), TRUE, &err))
@@ -1321,8 +1414,18 @@ _thunar_io_jobs_rename (ThunarJob  *job,
       exo_job_send_to_mainloop (EXO_JOB (job),
                                 _thunar_io_jobs_rename_notify,
                                 g_object_ref (file), g_object_unref);
+
+      if (log_mode == THUNAR_OPERATION_LOG_OPERATIONS)
+        {
+          operation = thunar_job_operation_new (THUNAR_JOB_OPERATION_KIND_RENAME);
+          thunar_job_operation_add (operation, g_file_new_for_uri (old_file_uri), thunar_file_get_file (file));
+          thunar_job_operation_history_commit (operation);
+          g_object_unref (operation);
+        }
     }
 
+  g_free (old_file_uri);
+
   /* abort on errors or cancellation */
   if (err != NULL)
     {
@@ -1336,13 +1439,15 @@ _thunar_io_jobs_rename (ThunarJob  *job,
 
 
 ThunarJob *
-thunar_io_jobs_rename_file (ThunarFile  *file,
-                            const gchar *display_name)
+thunar_io_jobs_rename_file (ThunarFile            *file,
+                            const gchar           *display_name,
+                            ThunarOperationLogMode log_mode)
 {
   _thunar_return_val_if_fail (THUNAR_IS_FILE (file), NULL);
   _thunar_return_val_if_fail (g_utf8_validate (display_name, -1, NULL), NULL);
 
-  return thunar_simple_job_new (_thunar_io_jobs_rename, 2,
+  return thunar_simple_job_new (_thunar_io_jobs_rename, 3,
                                 THUNAR_TYPE_FILE, file,
-                                G_TYPE_STRING, display_name);
+                                G_TYPE_STRING, display_name,
+                                THUNAR_TYPE_OPERATION_LOG_MODE, log_mode);
 }
