@@ -74,6 +74,7 @@ enum
   PROP_LAST_SORT_COLUMN,
   PROP_LAST_SORT_ORDER,
   PROP_LAST_STATUSBAR_VISIBLE,
+  PROP_LAST_IMAGE_PREVIEW_VISIBLE,
   PROP_LAST_VIEW,
   PROP_LAST_WINDOW_HEIGHT,
   PROP_LAST_WINDOW_WIDTH,
@@ -114,13 +115,17 @@ enum
   PROP_MISC_WINDOW_ICON,
   PROP_MISC_TRANSFER_USE_PARTIAL,
   PROP_MISC_TRANSFER_VERIFY_FILE,
+  PROP_MISC_IMAGE_PREVIEW_FULL,
   PROP_SHORTCUTS_ICON_EMBLEMS,
   PROP_SHORTCUTS_ICON_SIZE,
   PROP_TREE_ICON_EMBLEMS,
   PROP_TREE_ICON_SIZE,
   PROP_MISC_SWITCH_TO_NEW_TAB,
   PROP_MISC_VERTICAL_SPLIT_PANE,
+  PROP_MISC_OPEN_NEW_WINDOWS_IN_SPLIT_VIEW,
   PROP_MISC_COMPACT_VIEW_MAX_CHARS,
+  PROP_MISC_HIGHLIGHTING_ENABLED,
+  PROP_MISC_UNDO_REDO_HISTORY_SIZE,
   N_PROPERTIES,
 };
 
@@ -444,12 +449,13 @@ thunar_preferences_class_init (ThunarPreferencesClass *klass)
    * ThunarPreferences:last-toolbar-visible-buttons:
    *
    * The comma separated list that specifies the visibility of toolbar items.
+   * The order of the default value corresponds to the order in which the buttons are added inside 'thunar_window_location_toolbar_create'
    **/
     preferences_props[PROP_LAST_TOOLBAR_VISIBLE_BUTTONS] =
         g_param_spec_string ("last-toolbar-visible-buttons",
                              "LastToolbarVisibleButtons",
                              NULL,
-                             "0,1,1,1,1,0,0,0,0,0,0,1,0,1",
+                             "0,1,1,1,1,0,0,0,0,0,0,0,0,0,1,0,1",
                              EXO_PARAM_READWRITE);
 
   /**
@@ -461,7 +467,7 @@ thunar_preferences_class_init (ThunarPreferencesClass *klass)
         g_param_spec_string ("last-toolbar-item-order",
                              "LastToolbarButtonOrder",
                              NULL,
-                             "0,1,2,3,4,5,6,7,8,9,10,11,12,13",
+                             "0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16",
                              EXO_PARAM_READWRITE);
 
   /**
@@ -527,6 +533,18 @@ thunar_preferences_class_init (ThunarPreferencesClass *klass)
                             TRUE,
                             EXO_PARAM_READWRITE);
 
+  /**
+   * ThunarPreferences:last-image-preview-visible:
+   *
+   * Whether to display an image preview in new windows by default.
+   **/
+  preferences_props[PROP_LAST_IMAGE_PREVIEW_VISIBLE] =
+      g_param_spec_boolean ("last-image-preview-visible",
+                            "LastImagePreviewVisible",
+                            NULL,
+                            FALSE,
+                            EXO_PARAM_READWRITE);
+
   /**
    * ThunarPreferences:last-view:
    *
@@ -984,6 +1002,20 @@ thunar_preferences_class_init (ThunarPreferencesClass *klass)
                        THUNAR_VERIFY_FILE_MODE_DISABLED,
                        EXO_PARAM_READWRITE);
 
+  /**
+   * ThunarPreferences:misc-image-preview-mode:
+   *
+   * Whether the image preview functionality uses its own sidepane or is embedded
+   * in the left sidepane.
+   **/
+  preferences_props[PROP_MISC_IMAGE_PREVIEW_FULL] =
+    g_param_spec_enum ("misc-image-preview-mode",
+                       "MiscImagePreviewMoe",
+                       NULL,
+                       THUNAR_TYPE_IMAGE_PREVIEW_MODE,
+                       THUNAR_IMAGE_PREVIEW_MODE_EMBEDDED,
+                       EXO_PARAM_READWRITE);
+
   /**
    * ThunarPreferences:misc-confirm-close-multiple-tabs:
    *
@@ -1092,6 +1124,18 @@ thunar_preferences_class_init (ThunarPreferencesClass *klass)
                           FALSE,
                           EXO_PARAM_READWRITE);
 
+  /**
+   * ThunarPreferences:misc-open-new-windows-in-split-view
+   *
+   * If true, all thunar windows will have split view enabled.
+   **/
+  preferences_props[PROP_MISC_OPEN_NEW_WINDOWS_IN_SPLIT_VIEW] =
+    g_param_spec_boolean ("misc-open-new-windows-in-split-view",
+                          "MiscOpenNewWindowsInSplitView",
+                          NULL,
+                          FALSE,
+                          EXO_PARAM_READWRITE);
+
   /**
    * ThunarPreferences:compact-view-max-chars
    *
@@ -1105,6 +1149,33 @@ thunar_preferences_class_init (ThunarPreferencesClass *klass)
                         50,
                         EXO_PARAM_READWRITE);
 
+  /**
+   * ThunarPreferences:misc-highlighting-enabled
+   *
+   * If true file highlighting feature across the various views is enabled.
+   * Can be toggled using the View > Show File Highlight.
+   **/
+  preferences_props[PROP_MISC_HIGHLIGHTING_ENABLED] =
+      g_param_spec_boolean ("misc-highlighting-enabled",
+                            "MiscHighlightingEnabled",
+                            NULL,
+                            thunar_g_vfs_metadata_is_supported (),
+                            EXO_PARAM_READWRITE);
+
+  /**
+   * ThunarPreferences:misc-undo-redo-history-size
+   *
+   * Maximum number of ThunarJobOperations which can be undone/redone
+   * -1 for unlimited
+   **/
+  preferences_props[PROP_MISC_UNDO_REDO_HISTORY_SIZE] =
+      g_param_spec_int ("misc-undo-redo-history-size",
+                        "MiscUndoRedoHistorySize",
+                        NULL,
+                        -1, G_MAXINT,
+                        10,
+                        EXO_PARAM_READWRITE);
+
   /* install all properties */
   g_object_class_install_properties (gobject_class, N_PROPERTIES, preferences_props);
 }
