@@ -76,6 +76,15 @@ enum
   LAST_SIGNAL,
 };
 
+/* Notebook Pages */
+enum
+{
+  NOTEBOOK_PAGE_GENERAL,
+  NOTEBOOK_PAGE_EMBLEMS,
+  NOTEBOOK_PAGE_PERMISSIONS,
+  NOTEBOOK_PAGE_HIGHLIGHT
+};
+
 
 
 static void     thunar_properties_dialog_dispose              (GObject                     *object);
@@ -101,6 +110,17 @@ static void     thunar_properties_dialog_icon_button_clicked  (GtkWidget
 static void     thunar_properties_dialog_update               (ThunarPropertiesDialog      *dialog);
 static void     thunar_properties_dialog_update_providers     (ThunarPropertiesDialog      *dialog);
 static GList   *thunar_properties_dialog_get_files            (ThunarPropertiesDialog      *dialog);
+static void     thunar_properties_dialog_reset_highlight      (ThunarPropertiesDialog      *dialog);
+static void     thunar_properties_dialog_apply_highlight      (ThunarPropertiesDialog      *dialog);
+static void     thunar_properties_dialog_set_foreground       (ThunarPropertiesDialog      *dialog);
+static void     thunar_properties_dialog_set_background       (ThunarPropertiesDialog      *dialog);
+static void     thunar_properties_dialog_update_apply_button  (ThunarPropertiesDialog      *dialog);
+static void     thunar_properties_dialog_colorize_example_box (ThunarPropertiesDialog      *dialog,
+                                                               const gchar                 *background,
+                                                               const gchar                 *foreground);
+static void     thunar_properties_dialog_color_editor_changed (ThunarPropertiesDialog      *dialog);
+static void     thunar_properties_dialog_color_editor_close   (ThunarPropertiesDialog      *dialog);
+static void     thunar_properties_dialog_notebook_page_changed(ThunarPropertiesDialog      *dialog);
 
 
 struct _ThunarPropertiesDialogClass
@@ -143,6 +163,8 @@ struct _ThunarPropertiesDialog
   GtkWidget              *deleted_label;
   GtkWidget              *modified_label;
   GtkWidget              *accessed_label;
+  GtkWidget              *capacity_vbox;
+  GtkWidget              *capacity_label;
   GtkWidget              *freespace_vbox;
   GtkWidget              *freespace_bar;
   GtkWidget              *freespace_label;
@@ -151,6 +173,14 @@ struct _ThunarPropertiesDialog
   GtkWidget              *permissions_chooser;
   GtkWidget              *content_label;
   GtkWidget              *content_value_label;
+  GtkWidget              *color_chooser;
+  GtkWidget              *example_box;
+  GtkWidget              *highlight_buttons;
+  GtkWidget              *highlight_apply_button;
+  GtkWidget              *editor_button;
+
+  gchar                  *foreground_color;
+  gchar                  *background_color;
 };
 
 
@@ -237,6 +267,9 @@ thunar_properties_dialog_init (ThunarPropertiesDialog *dialog)
   GtkWidget *spacer;
   guint      row = 0;
   GtkWidget *image;
+  GtkWidget *button;
+  GtkWidget *infobar;
+  GtkWidget *frame;
 
   /* acquire a reference on the preferences and monitor the
      "misc-date-style" and "misc-file-size-binary" settings */
@@ -606,6 +639,28 @@ thunar_properties_dialog_init (ThunarPropertiesDialog *dialog)
 
   ++row;
 
+  label = gtk_label_new (_("Capacity:"));
+  gtk_label_set_attributes (GTK_LABEL (label), thunar_pango_attr_list_bold ());
+  gtk_label_set_xalign (GTK_LABEL (label), 1.0f);
+  gtk_label_set_yalign (GTK_LABEL (label), 0.0f);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, row, 1, 1);
+  gtk_widget_show (label);
+
+  dialog->capacity_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 4);
+  gtk_widget_set_hexpand (dialog->capacity_vbox, TRUE);
+  gtk_grid_attach (GTK_GRID (grid), dialog->capacity_vbox, 1, row, 1, 1);
+  g_object_bind_property (G_OBJECT (dialog->capacity_vbox), "visible",
+                          G_OBJECT (label),                 "visible",
+                          G_BINDING_SYNC_CREATE);
+  gtk_widget_show (dialog->capacity_vbox);
+
+  dialog->capacity_label = g_object_new (GTK_TYPE_LABEL, "xalign", 0.0f, NULL);
+  gtk_label_set_selectable (GTK_LABEL (dialog->capacity_label), TRUE);
+  gtk_box_pack_start (GTK_BOX (dialog->capacity_vbox), dialog->capacity_label, TRUE, TRUE, 0);
+  gtk_widget_show (dialog->capacity_label);
+
+  ++row;
+
   label = gtk_label_new (_("Usage:"));
   gtk_label_set_attributes (GTK_LABEL (label), thunar_pango_attr_list_bold ());
   gtk_label_set_xalign (GTK_LABEL (label), 1.0f);
@@ -663,6 +718,140 @@ thunar_properties_dialog_init (ThunarPropertiesDialog *dialog)
   gtk_notebook_append_page (GTK_NOTEBOOK (dialog->notebook), dialog->permissions_chooser, label);
   gtk_widget_show (dialog->permissions_chooser);
   gtk_widget_show (label);
+
+  /*
+     Highlight Color Chooser
+   */
+  grid = gtk_grid_new ();
+  label = gtk_label_new (_("Highlight"));
+  gtk_widget_set_halign (grid, GTK_ALIGN_CENTER);
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 12);
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
+  gtk_container_set_border_width (GTK_CONTAINER (grid), 12);
+  gtk_notebook_append_page (GTK_NOTEBOOK (dialog->notebook), grid, label);
+  gtk_widget_show (label);
+  gtk_widget_show (grid);
+
+  row = 0;
+
+  chooser = gtk_color_chooser_widget_new ();
+  dialog->color_chooser = chooser;
+  g_signal_connect_swapped (G_OBJECT (chooser), "notify::show-editor",
+                            G_CALLBACK (thunar_properties_dialog_color_editor_changed), dialog);
+  g_signal_connect_swapped (G_OBJECT (dialog->notebook), "switch-page",
+                            G_CALLBACK (thunar_properties_dialog_notebook_page_changed), dialog);
+  gtk_grid_attach (GTK_GRID (grid), chooser, 0, row, 1, 1);
+  gtk_widget_set_vexpand (chooser, TRUE);
+  gtk_widget_show (chooser);
+
+  row++;
+
+  /* check if gvfs metadata is supported */
+  if (G_UNLIKELY (!thunar_g_vfs_metadata_is_supported ()))
+    {
+      frame = g_object_new (GTK_TYPE_FRAME, "border-width", 0, "shadow-type", GTK_SHADOW_NONE, NULL);
+      gtk_grid_attach (GTK_GRID (grid), frame, 0, row, 1, 1);
+      gtk_widget_set_sensitive (dialog->color_chooser, FALSE);
+      gtk_widget_show (frame);
+
+      label = gtk_label_new (_("Missing dependencies"));
+      gtk_label_set_attributes (GTK_LABEL (label), thunar_pango_attr_list_bold ());
+      gtk_frame_set_label_widget (GTK_FRAME (frame), label);
+      gtk_widget_show (label);
+
+      infobar = gtk_info_bar_new ();
+      gtk_container_set_border_width (GTK_CONTAINER (infobar), 12);
+      label = gtk_label_new (NULL);
+      gtk_label_set_markup (GTK_LABEL (label), _("It looks like <a href=\"https://wiki.gnome.org/Projects/gvfs\">gvfs</a> is not available.\n"
+                                                 "This feature will not work. "
+                                                 "<a href=\"https://docs.xfce.org/xfce/thunar/unix-filesystem#gnome_virtual_file_system\">[Read more]</a>"));
+      box = gtk_info_bar_get_content_area (GTK_INFO_BAR (infobar));
+      gtk_container_add (GTK_CONTAINER (box), label);
+      gtk_info_bar_set_message_type (GTK_INFO_BAR (infobar), GTK_MESSAGE_WARNING);
+      gtk_widget_show (label);
+      gtk_widget_show (infobar);
+      gtk_container_add (GTK_CONTAINER (frame), infobar);
+
+      row++;
+    }
+  else
+    {
+      dialog->example_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+      gtk_widget_set_name (dialog->example_box, "example");
+      gtk_widget_set_hexpand (dialog->example_box, TRUE);
+      gtk_widget_set_margin_top (dialog->example_box, 10);
+      gtk_widget_set_margin_bottom (dialog->example_box, 10);
+      gtk_grid_attach (GTK_GRID (grid), dialog->example_box, 0, row, 1, 1);
+      gtk_widget_show (dialog->example_box);
+
+      image = gtk_image_new_from_icon_name ("text-x-generic", GTK_ICON_SIZE_DIALOG);
+      gtk_box_pack_start (GTK_BOX (dialog->example_box), image, FALSE, FALSE, 5);
+      gtk_widget_set_margin_top (image, 5);
+      gtk_widget_set_margin_bottom (image, 5);
+      gtk_widget_show (image);
+
+      label = gtk_label_new_with_mnemonic (_("Example.txt"));
+      gtk_box_pack_start (GTK_BOX (dialog->example_box), label, TRUE, TRUE, 0);
+      gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
+      gtk_widget_show (label);
+
+      row++;
+    }
+
+  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+  dialog->highlight_buttons = box;
+  gtk_widget_set_hexpand (box, TRUE);
+  gtk_widget_set_vexpand (box, TRUE);
+  gtk_grid_attach (GTK_GRID (grid), box, 0, row, 1, 1);
+  if (G_UNLIKELY (!thunar_g_vfs_metadata_is_supported ()))
+    gtk_widget_set_sensitive (box, FALSE);
+  gtk_widget_show (box);
+
+  button = gtk_button_new_with_mnemonic (_("_Apply"));
+  gtk_style_context_add_class (gtk_widget_get_style_context (button), "suggested-action");
+  g_signal_connect_swapped (G_OBJECT (button), "clicked",
+                            G_CALLBACK (thunar_properties_dialog_apply_highlight), dialog);
+  gtk_box_pack_end (GTK_BOX (box), button, FALSE, FALSE, 0);
+  gtk_widget_set_vexpand (button, FALSE);
+  gtk_widget_set_valign (button, GTK_ALIGN_END);
+  gtk_widget_show (button);
+
+  dialog->highlight_apply_button = button;
+  gtk_widget_set_sensitive (dialog->highlight_apply_button, FALSE);
+
+  button = gtk_button_new_with_mnemonic (_("_Reset"));
+  g_signal_connect_swapped (G_OBJECT (button), "clicked",
+                            G_CALLBACK (thunar_properties_dialog_reset_highlight), dialog);
+  gtk_box_pack_end (GTK_BOX (box), button, FALSE, FALSE, 0);
+  gtk_widget_set_vexpand (button, FALSE);
+  gtk_widget_set_valign (button, GTK_ALIGN_END);
+  gtk_widget_show (button);
+
+  button = gtk_button_new_with_mnemonic (_("Set _Foreground"));
+  g_signal_connect_swapped (G_OBJECT (button), "clicked",
+                            G_CALLBACK (thunar_properties_dialog_set_foreground), dialog);
+  gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
+  gtk_widget_set_vexpand (button, FALSE);
+  gtk_widget_set_valign (button, GTK_ALIGN_END);
+  gtk_widget_show (button);
+
+  button = gtk_button_new_with_mnemonic (_("Set _Background"));
+  g_signal_connect_swapped (G_OBJECT (button), "clicked",
+                            G_CALLBACK (thunar_properties_dialog_set_background), dialog);
+  gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
+  gtk_widget_set_vexpand (button, FALSE);
+  gtk_widget_set_valign (button, GTK_ALIGN_END);
+  gtk_widget_show (button);
+
+  box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+  dialog->editor_button = box;
+  gtk_grid_attach (GTK_GRID (grid), box, 0, row, 1, 1);
+
+  button = gtk_button_new_from_icon_name ("go-previous", GTK_ICON_SIZE_BUTTON);
+  g_signal_connect_swapped (G_OBJECT (button), "clicked",
+                            G_CALLBACK (thunar_properties_dialog_color_editor_close), dialog);
+  gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE, 0);
+  gtk_widget_show (button);
 }
 
 
@@ -863,7 +1052,7 @@ thunar_properties_dialog_name_activate (GtkWidget              *entry,
   old_name = thunar_file_get_display_name (file);
   if (g_utf8_collate (new_name, old_name) != 0)
     {
-      job = thunar_io_jobs_rename_file (file, new_name);
+      job = thunar_io_jobs_rename_file (file, new_name, THUNAR_OPERATION_LOG_OPERATIONS);
       exo_job_launch (EXO_JOB (job));
       if (job != NULL)
         {
@@ -1011,12 +1200,16 @@ thunar_properties_dialog_update_single (ThunarPropertiesDialog *dialog)
   gchar             *volume_name;
   gchar             *volume_id;
   gchar             *volume_label;
+  guint64            capacity = 0;
+  gchar             *capacity_str = NULL;
   ThunarFile        *file;
   ThunarFile        *parent_file;
   gboolean           show_chooser;
   guint64            fs_free;
   guint64            fs_size;
   gdouble            fs_fraction = 0.0;
+  const gchar       *background;
+  const gchar       *foreground;
 
   _thunar_return_if_fail (THUNAR_IS_PROPERTIES_DIALOG (dialog));
   _thunar_return_if_fail (g_list_length (dialog->files) == 1);
@@ -1030,7 +1223,7 @@ thunar_properties_dialog_update_single (ThunarPropertiesDialog *dialog)
 
   /* queue a new thumbnail request */
   thunar_thumbnailer_queue_file (dialog->thumbnailer, file,
-                                 &dialog->thumbnail_request);
+                                 &dialog->thumbnail_request, THUNAR_THUMBNAIL_SIZE_DEFAULT);
 
   icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (dialog)));
   icon_factory = thunar_icon_factory_get_for_icon_theme (icon_theme);
@@ -1232,6 +1425,12 @@ thunar_properties_dialog_update_single (ThunarPropertiesDialog *dialog)
       gtk_widget_hide (dialog->freespace_vbox);
     }
 
+  /* update the capacity (space of containing volume) */
+  if (thunar_g_file_get_free_space (thunar_file_get_file (file), NULL, &capacity))
+    capacity_str = g_format_size_full (capacity, dialog->file_size_binary ? G_FORMAT_SIZE_IEC_UNITS : G_FORMAT_SIZE_DEFAULT);
+  gtk_label_set_text (GTK_LABEL (dialog->capacity_label), capacity_str);
+  g_free (capacity_str);
+
   /* update the volume */
   volume = thunar_file_get_volume (file);
   if (G_LIKELY (volume != NULL))
@@ -1257,6 +1456,13 @@ thunar_properties_dialog_update_single (ThunarPropertiesDialog *dialog)
       gtk_widget_hide (dialog->volume_label);
     }
 
+  if (G_LIKELY (thunar_g_vfs_metadata_is_supported ()))
+    {
+      background = thunar_file_get_metadata_setting (file, "highlight-color-background");
+      foreground = thunar_file_get_metadata_setting (file, "highlight-color-foreground");
+      thunar_properties_dialog_colorize_example_box (dialog, background, foreground);
+    }
+
   /* cleanup */
   g_object_unref (G_OBJECT (icon_factory));
   g_free (date_custom_style);
@@ -1621,3 +1827,202 @@ thunar_properties_dialog_set_file (ThunarPropertiesDialog *dialog,
       thunar_properties_dialog_set_files (dialog, &foo);
     }
 }
+
+
+
+static void
+thunar_properties_dialog_reset_highlight (ThunarPropertiesDialog *dialog)
+{
+  GList   *lp;
+
+  _thunar_return_if_fail (THUNAR_IS_PROPERTIES_DIALOG (dialog));
+
+  for (lp = dialog->files; lp != NULL; lp = lp->next)
+    {
+      thunar_file_clear_metadata_setting (lp->data, "highlight-color-background");
+      thunar_file_clear_metadata_setting (lp->data, "highlight-color-foreground");
+    }
+
+  /* clear previouly set colors */
+  g_free (dialog->foreground_color);
+  dialog->foreground_color = NULL;
+  g_free (dialog->background_color);
+  dialog->background_color = NULL;
+
+  thunar_properties_dialog_colorize_example_box (dialog, NULL, NULL);
+
+  thunar_properties_dialog_reload (dialog);
+
+  thunar_properties_dialog_update_apply_button (dialog);
+}
+
+
+
+static void
+thunar_properties_dialog_apply_highlight (ThunarPropertiesDialog *dialog)
+{
+  GList    *lp;
+  gboolean  highlighting_enabled;
+
+  /* if this feature is disabled, then enable the feature */
+  if (dialog->foreground_color != NULL || dialog->background_color != NULL)
+    {
+      g_object_get (G_OBJECT (dialog->preferences), "misc-highlighting-enabled", &highlighting_enabled, NULL);
+      if (!highlighting_enabled)
+        g_object_set (G_OBJECT (dialog->preferences), "misc-highlighting-enabled", TRUE, NULL);
+    }
+
+  for (lp = dialog->files; lp != NULL; lp = lp->next)
+    {
+      if (dialog->foreground_color != NULL)
+        thunar_file_set_metadata_setting (lp->data, "highlight-color-foreground", dialog->foreground_color, FALSE);
+      if (dialog->background_color != NULL)
+        thunar_file_set_metadata_setting (lp->data, "highlight-color-background", dialog->background_color, FALSE);
+    }
+
+  thunar_properties_dialog_reload (dialog);
+
+  thunar_properties_dialog_update_apply_button (dialog);
+}
+
+
+
+static void
+thunar_properties_dialog_set_foreground (ThunarPropertiesDialog *dialog)
+{
+  GdkRGBA   color;
+  gchar    *color_str;
+
+  _thunar_return_if_fail (THUNAR_IS_PROPERTIES_DIALOG (dialog));
+
+  gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (dialog->color_chooser), &color);
+  color_str = gdk_rgba_to_string (&color);
+  thunar_properties_dialog_colorize_example_box (dialog, NULL, color_str);
+  g_free (dialog->foreground_color);
+  dialog->foreground_color = NULL;
+  dialog->foreground_color = color_str;
+  thunar_properties_dialog_update_apply_button (dialog);
+}
+
+
+
+static void
+thunar_properties_dialog_set_background (ThunarPropertiesDialog *dialog)
+{
+  GdkRGBA   color;
+  gchar    *color_str;
+
+  _thunar_return_if_fail (THUNAR_IS_PROPERTIES_DIALOG (dialog));
+
+  gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (dialog->color_chooser), &color);
+  color_str = gdk_rgba_to_string (&color);
+  thunar_properties_dialog_colorize_example_box (dialog, color_str, NULL);
+  g_free (dialog->background_color);
+  dialog->background_color = NULL;
+  dialog->background_color = color_str;
+  thunar_properties_dialog_update_apply_button (dialog);
+}
+
+
+
+static void
+thunar_properties_dialog_update_apply_button (ThunarPropertiesDialog *dialog)
+{
+  for (GList *lp = dialog->files; lp != NULL; lp = lp->next)
+    {
+      if (dialog->foreground_color != NULL && g_strcmp0 (dialog->foreground_color, thunar_file_get_metadata_setting (lp->data, "highlight-color-foreground")) != 0)
+        {
+          gtk_widget_set_sensitive (dialog->highlight_apply_button, TRUE);
+          return;
+        }
+
+      if (dialog->background_color != NULL && g_strcmp0 (dialog->background_color, thunar_file_get_metadata_setting (lp->data, "highlight-color-background")) != 0)
+        {
+          gtk_widget_set_sensitive (dialog->highlight_apply_button, TRUE);
+          return;
+        }
+    }
+
+  gtk_widget_set_sensitive (dialog->highlight_apply_button, FALSE);
+}
+
+
+
+static void
+thunar_properties_dialog_colorize_example_box (ThunarPropertiesDialog *dialog,
+                                               const gchar            *background,
+                                               const gchar            *foreground)
+{
+  GtkCssProvider *provider = gtk_css_provider_new ();
+  gchar          *css_data = NULL;
+
+  _thunar_return_if_fail (THUNAR_IS_PROPERTIES_DIALOG (dialog));
+
+  gtk_widget_reset_style (dialog->example_box);
+
+  if (background != NULL && foreground != NULL)
+    css_data = g_strdup_printf ("#example { background-color: %s; color: %s; }", background, foreground);
+  else if (background != NULL)
+    css_data = g_strdup_printf ("#example { background-color: %s; }", background);
+  else if (foreground != NULL)
+    css_data = g_strdup_printf ("#example { color: %s; }", foreground);
+  else
+    css_data = g_strdup_printf ("#example { color: inherit; background-color: inherit; }");
+
+  if (css_data == NULL)
+    return;
+
+  gtk_css_provider_load_from_data (provider, css_data, -1, NULL);
+  gtk_style_context_add_provider (gtk_widget_get_style_context (dialog->example_box), GTK_STYLE_PROVIDER (provider), GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+
+  gtk_widget_show (dialog->example_box);
+  g_free (css_data);
+}
+
+
+
+static void
+thunar_properties_dialog_color_editor_changed (ThunarPropertiesDialog *dialog)
+{
+  gboolean show_editor;
+
+  _thunar_return_if_fail (THUNAR_IS_PROPERTIES_DIALOG (dialog));
+
+  g_object_get (dialog->color_chooser, "show-editor", &show_editor, NULL);
+  
+  if (show_editor)
+    {
+      gtk_widget_show (dialog->editor_button);
+      gtk_widget_hide (dialog->example_box);
+      gtk_widget_hide (dialog->highlight_buttons);
+    }
+  else
+    {
+      gtk_widget_hide (dialog->editor_button);
+      gtk_widget_show (dialog->example_box);
+      gtk_widget_show (dialog->highlight_buttons);
+    }
+}
+
+
+
+static void
+thunar_properties_dialog_color_editor_close (ThunarPropertiesDialog *dialog)
+{
+  GdkRGBA color;
+
+  gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (dialog->color_chooser), &color);
+  gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (dialog->color_chooser), &color);
+
+  g_object_set (G_OBJECT (dialog->color_chooser), "show-editor", FALSE, NULL);
+}
+
+
+
+static void
+thunar_properties_dialog_notebook_page_changed (ThunarPropertiesDialog *dialog)
+{
+  /* if the highlight tab is not active then the color editor should be switched off */
+  if (gtk_notebook_get_current_page (GTK_NOTEBOOK (dialog->notebook)) != NOTEBOOK_PAGE_HIGHLIGHT)
+    g_object_set (G_OBJECT (dialog->color_chooser), "show-editor", FALSE, NULL);
+}
