@@ -31,13 +31,12 @@
 #include <libxfce4ui/libxfce4ui.h>
 
 #include <thunar/thunar-abstract-dialog.h>
+#include <thunar/thunar-action-manager.h>
 #include <thunar/thunar-application.h>
 #include <thunar/thunar-dialogs.h>
 #include <thunar/thunar-gtk-extensions.h>
 #include <thunar/thunar-icon-factory.h>
 #include <thunar/thunar-icon-renderer.h>
-#include <thunar/thunar-launcher.h>
-#include <thunar/thunar-launcher.h>
 #include <thunar/thunar-menu.h>
 #include <thunar/thunar-private.h>
 #include <thunar/thunar-properties-dialog.h>
@@ -162,7 +161,7 @@ struct _ThunarRenamerDialog
 
   ThunarRenamerModel  *model;
 
-  ThunarLauncher      *launcher;
+  ThunarActionManager *action_mgr;
 
   GtkWidget           *cancel_button;
   GtkWidget           *close_button;
@@ -376,10 +375,10 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
   gtk_dialog_set_default_response (GTK_DIALOG (renamer_dialog), GTK_RESPONSE_APPLY);
   gtk_widget_set_tooltip_text (button, _("Click here to actually rename the files listed above to their new names and keep the window open."));
 
-  /* setup the launcher support for this dialog */
-  renamer_dialog->launcher = g_object_new (THUNAR_TYPE_LAUNCHER, "widget", GTK_WIDGET (renamer_dialog), NULL);
-  g_object_bind_property (G_OBJECT (renamer_dialog), "selected-files", G_OBJECT (renamer_dialog->launcher), "selected-files", G_BINDING_SYNC_CREATE);
-  g_object_bind_property (G_OBJECT (renamer_dialog), "current-directory", G_OBJECT (renamer_dialog->launcher), "current-directory", G_BINDING_SYNC_CREATE);
+  /* setup the action manager support for this dialog */
+  renamer_dialog->action_mgr = g_object_new (THUNAR_TYPE_ACTION_MANAGER, "widget", GTK_WIDGET (renamer_dialog), NULL);
+  g_object_bind_property (G_OBJECT (renamer_dialog), "selected-files", G_OBJECT (renamer_dialog->action_mgr), "selected-files", G_BINDING_SYNC_CREATE);
+  g_object_bind_property (G_OBJECT (renamer_dialog), "current-directory", G_OBJECT (renamer_dialog->action_mgr), "current-directory", G_BINDING_SYNC_CREATE);
 
   /* add the toolbar to the dialog */
   toolbar = gtk_toolbar_new ();
@@ -524,7 +523,7 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
       active_str = xfce_rc_read_entry_untranslated (rc, "LastActiveMode", "");
       for (active = 0, n = 0; n < klass->n_values; ++n)
         {
-          if (exo_str_is_equal (active_str, klass->values[n].value_name))
+          if (g_strcmp0 (active_str, klass->values[n].value_name) == 0)
             active = n;
           gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (mcombo), _(klass->values[n].value_nick));
         }
@@ -553,7 +552,7 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
           gtk_notebook_append_page (GTK_NOTEBOOK (notebook), lp->data, NULL);
 
           /* check if this page should be active by default */
-          if (exo_str_is_equal (G_OBJECT_TYPE_NAME (lp->data), active_str))
+          if (g_strcmp0 (G_OBJECT_TYPE_NAME (lp->data), active_str) == 0)
             active = g_list_position (renamers, lp);
 
           /* try to load the settings for the renamer */
@@ -653,8 +652,8 @@ thunar_renamer_dialog_finalize (GObject *object)
 {
   ThunarRenamerDialog *renamer_dialog = THUNAR_RENAMER_DIALOG (object);
 
-  /* release the launcher support */
-  g_object_unref (G_OBJECT (renamer_dialog->launcher));
+  /* release the action manager support */
+  g_object_unref (G_OBJECT (renamer_dialog->action_mgr));
 
   /* release our bulk rename model */
   g_object_unref (G_OBJECT (renamer_dialog->model));
@@ -899,7 +898,7 @@ thunar_renamer_dialog_context_menu (ThunarRenamerDialog *renamer_dialog)
   g_object_ref (G_OBJECT (renamer_dialog));
 
   menu = g_object_new (THUNAR_TYPE_MENU, "menu-type", THUNAR_MENU_TYPE_CONTEXT_RENAMER,
-                                         "launcher", renamer_dialog->launcher,
+                                         "action_mgr", renamer_dialog->action_mgr,
                                          "tab-support-disabled", TRUE,
                                          "change_directory-support-disabled", TRUE, NULL);
   thunar_menu_add_sections (menu, THUNAR_MENU_SECTION_OPEN | THUNAR_MENU_SECTION_SENDTO);
@@ -922,7 +921,7 @@ thunar_renamer_dialog_context_menu (ThunarRenamerDialog *renamer_dialog)
   thunar_renamer_dialog_append_menu_item (renamer_dialog, GTK_MENU_SHELL (menu), THUNAR_RENAMER_ACTION_REMOVE_FILES);
   xfce_gtk_menu_append_separator (GTK_MENU_SHELL (menu));
   thunar_menu_add_sections (menu, THUNAR_MENU_SECTION_PROPERTIES);
-  thunar_menu_hide_accel_labels (menu);
+  thunar_gtk_menu_hide_accel_labels (GTK_MENU (menu));
   gtk_widget_show_all (GTK_WIDGET (menu));
 
   thunar_gtk_menu_run (GTK_MENU (menu));
@@ -1638,7 +1637,7 @@ thunar_renamer_dialog_row_activated (GtkTreeView         *tree_view,
 {
   _thunar_return_if_fail (THUNAR_IS_RENAMER_DIALOG (renamer_dialog));
 
-  thunar_launcher_activate_selected_files (renamer_dialog->launcher, THUNAR_LAUNCHER_OPEN_AS_NEW_WINDOW, NULL);
+  thunar_action_manager_activate_selected_files (renamer_dialog->action_mgr, THUNAR_ACTION_MANAGER_OPEN_AS_NEW_WINDOW, NULL);
 }
 
 
